package ShowXPath;

import ShowXPath.TreeNode.*;
import ShowXPath.StackNode.*;
import ShowXPath.XMLParser.*;
import ShowXPath.GUI.*;

import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

class DocumentTree
{
	private SaxTree _saxTree;
	private JScrollPane _scrollpane;
	private XMLTree _xmltree;

	public DocumentTree(SaxTree saxTree, JScrollPane scrollpane,
											XMLTree xmltree)
	{
		_saxTree = saxTree;
		_scrollpane = scrollpane;
		_xmltree = xmltree;
	}

	public SaxTree getSaxTree()
	{
		return _saxTree;
	}

	public XMLTree getXMLTree()
	{
		return _xmltree;
	}

	public JScrollPane getScrollPane()
	{
		return _scrollpane;
	}
}


class DriverFrame extends JFrame
{
	private JScrollPane scrollpane;
	private Main _mainFrame;
	private XMLTree tree;
	private LinkedList _documents = new LinkedList();
	private ContextTree _contextTree;

	private JList _stackList;
	private JTextArea _errorField = new JTextArea("No Error", 1, 30);
	private JTextArea _valueField = new JTextArea(" ", 5, 30);
	private JTextArea _stepField = new JTextArea(" ", 5, 30);
	private LNodeset empty = new LNodeset();
	private Executor _exec = null;
	private Thread _thread = null;
	private String _debugStep = "No Stepping";
	private String _debugSteps[] = { "No Stepping",
																	 "Big Steps",
																	 "Small Steps",
																	 "Baby Steps"
	};
	private JComboBox _debugCombo = new JComboBox(_debugSteps);
	private JTabbedPane _tabbedpane;
	private Hashtable _variables = new Hashtable();
	private LRoot _root;

	private void print(String string)
	{
		//System.out.println("DriverFrame: " + string);
	}
	
	public void setExecutor(Executor exec)
	{
		print("Setting executor in driverframe");
		_exec = exec;
	}

	public void parserError(String message)
	{
		String mesg = message.replace('\t', ' ').replace('\r', ' ').replace('\n', ' ');
		_errorField.setText(mesg);
	}

	public void showNodes(LStackNode node)
	{
		if (node instanceof LNodeset)
		{
			tree.showNodes(node);
			for (int i = 1; i < _tabbedpane.getComponentCount(); i++)
			{
				XMLTree t = ((DocumentTree)_documents.get(i)).getXMLTree();
				t.showNodes(node);
			}
		}
		String s;
		if (node instanceof LNumber)
		{
			s = "Number: " + node.getStringValue();
		}
		else if (node instanceof LBoolean)
		{
			s = "Boolean: " + node.getStringValue();
		}
		else if (node instanceof LMark)
		{
			s = "Mark";
		}
		else if (node instanceof LNodeset)
		{
			String string = "Nodeset: ";
			Iterator ni = ((LNodeset)node).iterator();
			while (ni.hasNext())
			{
				Object member = ni.next();
				if (member instanceof LTreeNode)
				{
					string = string + "[node:" + ((LTreeNode)member).getID() + "]";
				}
				else if (member instanceof LStackNode)
				{
					string = string + "[stack:" + ((LStackNode)member).getStringValue() + "]";
				}
			}
			s = string;
		}
		else if (node instanceof LQname)
		{
			s = "QName: " + node.getStringValue();
		}
		else if (node instanceof LString)
		{
			s = "String: " + node.getStringValue();
		}
		else
		{
			s = "Error: " + node.toString();
		}
		_valueField.setText(s);
	}
	
	public Dimension getMinimumSize()
	{
		return getPreferredSize();
	}

	public Dimension getMaximumSize()
	{
		return getPreferredSize();
	}

	public void newPath(String p)
	{
		_mainFrame.setPath(p);
	}

	public void setThread(Thread t)
	{
		_thread = t;
	}

	public void newTree(String k)
	{
		// DOM2Internal dom = new DOM2Internal(k);
		// LRoot root = dom.convert();
		LTreeNode.resetID();
		SaxTree saxTree = new SaxTree(k);
		_root = saxTree.getRoot();
		_mainFrame.setRoot(_root);
		_variables = new Hashtable();
		this.setTree(_root, saxTree);
	}

	public void setDocument(SaxTree saxTree)
	{
		JScrollPane sp = new JScrollPane();
		XMLTree tree = new XMLTree(saxTree.getRoot(), _mainFrame);
		sp.setViewportView(tree);
		sp.setPreferredSize(new Dimension(350, 300));
		sp.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder("Tree Representation for the XML File"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
		DocumentTree dt = new DocumentTree(saxTree, sp, tree);
		_documents.add(dt);
		_tabbedpane.addTab(saxTree.getRoot().getURI(), sp);
	}

	public JFrame getThis()
	{
		return (JFrame) this;
	}
	
	public DriverFrame(Main mainFrame, String title)
	{
		setTitle(title);
		_mainFrame = mainFrame;
		_contextTree = new ContextTree(_mainFrame);
		
		addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent event)
				{
					dispose();	  // free the system resources
					System.exit(0); // close the application
				}
			});
		JMenuBar menuBar = new JMenuBar();
		JMenu variableMenu = new JMenu("Variables");
		JMenuItem newVariable = new JMenuItem("New Variable");
		newVariable.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent as)
				{
					print("New Variable");
					String varName = (String) JOptionPane.showInputDialog(getThis(), "Set the result to variable",
																			"Set New Variable",
																			JOptionPane.QUESTION_MESSAGE,
																			null, null, "");
					if (varName != null && !varName.equals(""))
					{
						_mainFrame.setVariable(varName);
					}
				}
			}
																	);
		variableMenu.add(newVariable);
		
		JMenuItem removeVariable = new JMenuItem("Remove Variable");
		removeVariable.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent as)
				{
					print("remove Variable");
				}
			}
																	);
		variableMenu.add(removeVariable);

		JMenu keyMenu = new JMenu("Keys");
		menuBar.add(variableMenu);
		//menuBar.add(keyMenu);

		this.setJMenuBar(menuBar);
		this.getContentPane().setLayout(new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));
		final JPanel topPanel = new JPanel();
		topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
		topPanel.add(new JLabel("File:"));

		final JTextField fileField = new JTextField("Select an XML file first", 10);
		fileField.setEditable(false);
		
		topPanel.add(fileField);

		_debugCombo.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent ae)
				{
					print("actionPerformed");
					setDebugStep((String)((JComboBox)ae.getSource()).getSelectedItem());
				}
				
			});
		final JPanel middlePanel = new JPanel();
		middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.X_AXIS));

		final JTextField exprField = new JTextField(40);
		exprField.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent ae)
				{
					String text = exprField.getText().trim();
					if (!text.equals(""))
					{
						newPath(exprField.getText());
						for (int i = _tabbedpane.getComponentCount() - 1; i > 0; i--)
						{
							_tabbedpane.removeTabAt(i);
							_documents.removeLast();
						}
						_mainFrame.clearIncludedDocuments();
						tree.showNodes(empty);
					}
				}
			});

		JButton currentFile = new JButton("Browse");

		currentFile.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent ae)
				{
					String wDir = System.getProperties().getProperty("user.dir");
					JFileChooser chooser = new JFileChooser(wDir);
					if (chooser.showOpenDialog(topPanel) == JFileChooser.APPROVE_OPTION)
					{
						fileField.setText(chooser.getSelectedFile().getPath());
						_tabbedpane.setTitleAt(0, fileField.getText());
						newTree(chooser.getSelectedFile().getPath());
						exprField.setEditable(true);
					}
				}
			});
		topPanel.add(currentFile);
		topPanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("XML"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));

		middlePanel.add(new JLabel("Expression:"));

		exprField.setEditable(false);
		middlePanel.add(exprField);

		_valueField.setLineWrap(true);
		_stepField.setLineWrap(true);

		final JButton startButton = new JButton("Start");
		startButton.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent ae)
				{
					String text = exprField.getText().trim();
					if (!text.equals(""))
					{
						newPath(exprField.getText());
						for (int i = _tabbedpane.getComponentCount() - 1; i > 0; i--)
						{
							_tabbedpane.removeTabAt(i);
							_documents.removeLast();
						}
						_mainFrame.clearIncludedDocuments();
						_valueField.setText(" ");
						_stepField.setText(" ");
						_errorField.setText("No Error");
						//_contextTree.removeAll();
						tree.showNodes(empty);
					}
				}
			});
		
		middlePanel.add(startButton);

		JButton clearField = new JButton("Clear");
		clearField.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent ae)
				{
					exprField.setText("");
					_errorField.setText("No Error");
					_stepField.setText(" ");
					_valueField.setText(" ");
					_contextTree.removeAll();
					_stackList.setListData(new Vector());
					_mainFrame.clearIncludedDocuments();
					for (int i = _tabbedpane.getComponentCount() - 1; i > 0; i--)
					{
						_tabbedpane.removeTabAt(i);
						_documents.removeLast();
					}
					tree.showNodes(empty);
				}
			});
		
		middlePanel.add(clearField);

		JButton stepButton = new JButton("Next Step");
		stepButton.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent ae)
				{
					if (_thread != null)
					{
						synchronized(_thread)
						{
							_thread.notify();
						}
					}
				}
			});
		middlePanel.add(stepButton);
		middlePanel.add(_debugCombo);
		middlePanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("XPath"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
		topPanel.setVisible(true);
		middlePanel.setVisible(true);
		
		this.getContentPane().add(topPanel);
		this.getContentPane().add(middlePanel);

		final JPanel treePanel = new JPanel();
		treePanel.setLayout(new BoxLayout(treePanel, BoxLayout.X_AXIS));
		
		scrollpane = new JScrollPane();
		tree = new XMLTree(_root = new LRoot(), _mainFrame);
		scrollpane.setViewportView(tree);
		scrollpane.setPreferredSize(new Dimension(350, 300));
		scrollpane.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder("Tree Representation for the XML File"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
		_tabbedpane = new JTabbedPane();
		_tabbedpane.addTab("No File", scrollpane);
		DocumentTree dt = new DocumentTree(new SaxTree(), scrollpane, tree);
		_documents.add(dt);
		treePanel.add(_tabbedpane);
		treePanel.setVisible(true);

		JScrollPane contextScrollPane = new JScrollPane();
		contextScrollPane.setViewportView(_contextTree);
		contextScrollPane.setPreferredSize(new Dimension(350, 300));
		contextScrollPane.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder("Context Tree for the XPath Expression"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
		treePanel.add(contextScrollPane);

		_stackList = new JList();
		JScrollPane stackScrollPane = new JScrollPane();
		stackScrollPane.setViewportView(_stackList);
		stackScrollPane.setPreferredSize(new Dimension(150, 300));
		stackScrollPane.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder("Selected Context Stack"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
		treePanel.add(stackScrollPane);

		this.getContentPane().add(treePanel);

		final JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.Y_AXIS));
		bottomPanel.add(_errorField);
		_errorField.setEditable(false);
		_errorField.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("Error Message"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
		JScrollPane stepScrollPane = new JScrollPane();
		stepScrollPane.setViewportView(_stepField);
		_stepField.setEditable(false);
		_stepField.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("Steps"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
		bottomPanel.add(stepScrollPane);
		JScrollPane valueScrollPane = new JScrollPane();
		valueScrollPane.setViewportView(_valueField);
		_valueField.setEditable(false);
		valueScrollPane.setVisible(true);
		valueScrollPane.setPreferredSize(new Dimension(800, 100));
		bottomPanel.add(valueScrollPane);
		valueScrollPane.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("Value of the Expression"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
		bottomPanel.setVisible(true);
		this.getContentPane().add(bottomPanel);
		this.getContentPane().setSize(800, 400);
		this.setSize(700, 400);
		this.pack();
		this.setVisible(true);
	}

	public void setStep(String string)
	{
		_stepField.setText(string);
	}

	public void setTree(LRoot root, SaxTree saxtree)
	{
		tree.showNodes(empty);
		for (int i = _tabbedpane.getComponentCount() - 1; i > 0; i--)
		{
			_tabbedpane.removeTabAt(i);
			_documents.removeLast();
		}
		if (_documents.size() > 0)
		{
			_documents.removeLast();
		}
		_root= root;
		tree = new XMLTree(root, _mainFrame);
		scrollpane.setViewportView(tree);
		tree.setVisible(true);
		tree.repaint();
		scrollpane.setSize(700, 500);
		scrollpane.setVisible(true);
		DocumentTree dt = new DocumentTree(saxtree, scrollpane, tree);
		_documents.add(dt);
//		_tabbedpane.addTab(saxtree.getRoot().getURI(), scrollpane);
		_tabbedpane.repaint();
//		scrollpane.setPreferredSize(new Dimension(700, 500));
		scrollpane.repaint();
		this.setVisible(true);
		this.repaint();
	}

	public String getDebugStep()
	{
		return _debugStep;
	}

	private void setDebugStep(String string)
	{
		_debugStep = string;
	}

	public ContextTree getContextTree()
	{
		return _contextTree;
	}

	public void showStack(Stack s)
	{
		Iterator i = s.iterator();
		Vector v = new Vector();
		while (i.hasNext())
		{
			LStackNode sn = (LStackNode)i.next();
			if (sn instanceof LNumber)
			{
				v.add("Number: " + sn.getStringValue());
			}
			else if (sn instanceof LBoolean)
			{
				v.add("Boolean: " + sn.getStringValue());
			}
			else if (sn instanceof LMark)
			{
				v.add("Mark");
			}
			else if (sn instanceof LNodeset)
			{
				String string = "Nodeset: ";
				Iterator ni = ((LNodeset)sn).iterator();
				while (ni.hasNext())
				{
					Object member = ni.next();
					if (member instanceof LTreeNode)
					{
						string = string + "[" + ((LTreeNode)member).getID() + "]";
					}
					else if (member instanceof LStackNode)
					{
						string = string + "[" + ((LStackNode)member).getStringValue() + "]";
					}
				}
				v.add(string);
			}
			else if (sn instanceof LQname)
			{
				v.add("QName: " + sn.getStringValue());
			}
			else if (sn instanceof LString)
			{
				v.add("String: " + sn.getStringValue());
			}
			else
			{
				v.add("Error: " + sn.toString());
			}
		}
		_stackList.setListData(v);
	}

}

