package ShowXPath;

import ShowXPath.XPath.node.*;
import ShowXPath.XPath.lexer.*;
import ShowXPath.XPath.parser.*;
import ShowXPath.GUI.*;
import ShowXPath.TreeNode.*;
import ShowXPath.StackNode.*;
import ShowXPath.XMLParser.*;

import java.io.*;
import java.util.*;
import java.util.Stack;
import java.net.URL;

public class Main
{
	private LRoot root = new LRoot();
	private Executor exec = null;
	private LTreeNode _context = null;
	private DriverFrame df;
	private RunXPath _rp;
	private Context _contextDisplay = null;
	private Context _rootContextDisplay = null;
	private Hashtable _documents = new Hashtable();
	private Hashtable _variables = new Hashtable();
	private LStackNode _currentNode = null;

	private void print(String string)
	{
		//System.out.println("In Main: " + string);
	}

	public static void main(String args[])
	{
		Main m = new Main();
		Context.setMain(m);
	}

	public void setRoot(LRoot _root)
	{
		root = _root;
		_context = root;
		_documents = new Hashtable();
		_documents.put(root.getURI(), root);
	}

	public void setContext(LTreeNode node)
	{
		if (exec != null)
		{
			exec.setContext(node);
			exec.setContextTree(df.getContextTree());
		}
		_context = node;
	}

	public void setStep(String s)
	{
		df.setStep(s);
	}

	public void setPath(String p)
	{
		df.parserError("No Error");
		try
		{
			NewLexer lexer = new NewLexer(
				new PushbackReader(
					new BufferedReader(
						new StringReader(p), 1024)));

			Parser parser = new Parser(lexer);

			Node ast = parser.parse();
			if (exec != null)
			{
				exec.cleanUp();
			}
			exec = new Executor(root, this, _variables);
			exec.setDebugStep(df.getDebugStep());
			df.setExecutor(exec);
			exec.setContext(_context);
			exec.setContextTree(df.getContextTree());
			_rp = new RunXPath(ast, exec);
			_rp.start();
			df.setThread(_rp);
		}
		catch (ParserException pe)
		{
			df.parserError(pe.getMessage());
		}
		catch (Exception e)
		{
			print(e.toString());
		}
	}

	public ContextTree getContextTree()
	{
		if (df != null)
		{
			return df.getContextTree();
		}
		return null;
	}
	

	public void showNodes(LStackNode node)
	{
		_currentNode = node;
		df.showNodes(node);
	}
	
	public Main()
	{
		df = new DriverFrame(this, "XPath Visualizer");
		df.setVisible(true);
		_context = root;
	}

	public void setContextDisplay(Context context)
	{
		_contextDisplay = context;
		LStackNode sn = context.stackRoot();
		Stack s = context.getStack();
		if (sn != null)
		{
			showNodes(sn);
		}
		showStack(s);
	}

	public void displayContext()
	{
		LStackNode sn = _contextDisplay.stackRoot();
		Stack s = _contextDisplay.getStack();
		if (sn != null)
		{
			showNodes(sn);
		}
		showStack(s);
	}

	public void updateContextDisplay(Context context)
	{
		if (_contextDisplay == context)
		{
			_contextDisplay = _rootContextDisplay;
			LStackNode sn = _contextDisplay.stackRoot();
			Stack s = _contextDisplay.getStack();
			if (sn != null)
			{
				showNodes(sn);
			}
			showStack(s);
		}
	}

	public void setRootContextDisplay(Context context)
	{
		_rootContextDisplay = context;
		_contextDisplay = context;
		LStackNode sn = context.stackRoot();
		Stack s = context.getStack();
		if (sn != null)
		{
			showNodes(sn);
		}
		showStack(s);
	}

	public Context getContextDisplay()
	{
		return _contextDisplay;
	}

	public void showStack(Stack s)
	{
		if (df != null)
		{
			df.showStack(s);
		}
	}

	public void clearVariables()
	{
		Enumeration keys = _variables.keys();
		for (; keys.hasMoreElements(); )
		{
			String key = (String) keys.nextElement();
			LStackNode node = (LStackNode) _variables.get(key);
			if (node instanceof LNodeset)
			{
				ListIterator i = ((LNodeset)node).listIterator();
				while (i.hasNext())
				{
					LTreeNode tn = (LTreeNode)i.next();
					if (!tn.getURI().equals(root.getURI()))
					{
						_variables.remove(key);
						break;
					}
				}
			}
		}
	}

	public void setVariable(String name)
	{
		if (_currentNode == null)
		{
			_variables.put(name, new LNodeset());
		}
		else
		{
			_variables.put(name, _currentNode);
		}
	}

	public void clearIncludedDocuments()
	{
		_documents = new Hashtable();
		_documents.put(root.getURI(), root);
		clearVariables();
	}

	public LRoot setDocument(URL uri)
	{
		if (_documents.get(uri.toString()) != null)
		{
			return (LRoot) _documents.get(uri.toString());
		}
		else
		{
			SaxTree saxtree = new SaxTree(uri);
			if (saxtree.getRoot() == null)
			{
				return null;
			}
			else if (df != null)
			{
				df.setDocument(saxtree);
			}
			return saxtree.getRoot();
		}
	}
}
