package ShowXPath.StackNode;

import java.util.*;
import java.io.*;

import ShowXPath.TreeNode.*;

public class LNumber implements LStackNode
{
	private double _value;
	private final static String _type = "LNumber";

	public String type()
	{
		return _type;
	}

	public LNumber(String number)
	{
		Double d;
		try
		{
			d = new Double(number);
			_value = d.doubleValue();
		}
		catch (NumberFormatException e)
		{
			_value = Double.NaN;
		}
	}

	public LNumber(double number)
	{
		_value = number;
	}

	public double getValue()
	{
		return _value;
	}

	public String toString()
	{
		return new Double(_value).toString();
	}

	public String getStringValue()
	{
		return new Double(_value).toString();
	}

	public LBoolean toLBoolean()
	{
		if (!Double.isNaN(_value) && _value != 0)
		{
			return new LBoolean(true);
		}
		else
		{
			return new LBoolean(false);
		}
	}

	public LString toLString()
	{
		return new LString(Double.toString(_value));
	}

	public LNumber toLNumber()
	{
		return this;
	}
	
	public boolean equals(LStackNode node)
	{
		if (node instanceof LNodeset)
		{
			ListIterator i = ((LNodeset)node).listIterator();

			while (i.hasNext())
			{
				LTreeNode n = (LTreeNode)i.next();
				double nnumber = new LString(n.toString()).toLNumber().getValue();
				
				if (_value == nnumber)
				{
					return true;
				}
			}
			return false;
		}
		else if (node instanceof LBoolean)
		{
			return toLBoolean().getValue() ==
				((LBoolean)node).getValue();
		}
		else if (node instanceof LString)
		{
			return _value ==
				((LString)node).toLNumber().getValue();
		}
		else if (node instanceof LNumber)
		{
			return _value ==
				((LNumber)node).getValue();
		}
		else
		{
			return false;
		}
	}
	
	public boolean less(LStackNode node)
	{
		if (node instanceof LNodeset)
		{
			ListIterator i = ((LNodeset)node).listIterator();

			while (i.hasNext())
			{
				LTreeNode n = (LTreeNode)i.next();
				double nnumber = new LString(n.toString()).toLNumber().getValue();
				
				if (_value < nnumber)
				{
					return true;
				}
			}
			return false;
		}
		else if (node instanceof LBoolean)
		{
			return _value <
				((LBoolean)node).toLNumber().getValue();
		}
		else if (node instanceof LString)
		{
			return _value <
				((LString)node).toLNumber().getValue();
		}
		else if (node instanceof LNumber)
		{
			return _value <
				((LNumber)node).getValue();
		}
		else
		{
			return false;
		}
	}

	public boolean greater(LStackNode node)
	{
		if (node instanceof LNodeset)
		{
			ListIterator i = ((LNodeset)node).listIterator();

			while (i.hasNext())
			{
				LTreeNode n = (LTreeNode)i.next();
				double nnumber = new LString(n.toString()).toLNumber().getValue();
				
				if (_value > nnumber)
				{
					return true;
				}
			}
			return false;
		}
		else if (node instanceof LBoolean)
		{
			return _value >
				((LBoolean)node).toLNumber().getValue();
		}
		else if (node instanceof LString)
		{
			return _value >
				((LString)node).toLNumber().getValue();
		}
		else if (node instanceof LNumber)
		{
			return _value >
				((LNumber)node).getValue();
		}
		else
		{
			return false;
		}
	}

	public boolean lessEquals(LStackNode node)
	{
		if (node instanceof LNodeset)
		{
			ListIterator i = ((LNodeset)node).listIterator();

			while (i.hasNext())
			{
				LTreeNode n = (LTreeNode)i.next();
				double nnumber = new LString(n.toString()).toLNumber().getValue();
				
				if (_value <= nnumber)
				{
					return true;
				}
			}
			return false;
		}
		else if (node instanceof LBoolean)
		{
			return _value <=
				((LBoolean)node).toLNumber().getValue();
		}
		else if (node instanceof LString)
		{
			return _value <=
				((LString)node).toLNumber().getValue();
		}
		else if (node instanceof LNumber)
		{
			return _value <=
				((LNumber)node).getValue();
		}
		else
		{
			return false;
		}
	}

	public boolean greaterEquals(LStackNode node)
	{
		if (node instanceof LNodeset)
		{
			ListIterator i = ((LNodeset)node).listIterator();

			while (i.hasNext())
			{
				LTreeNode n = (LTreeNode)i.next();
				double nnumber = new LString(n.toString()).toLNumber().getValue();
				
				if (_value >= nnumber)
				{
					return true;
				}
			}
			return false;
		}
		else if (node instanceof LBoolean)
		{
			return _value >=
				((LBoolean)node).toLNumber().getValue();
		}
		else if (node instanceof LString)
		{
			return _value >=
				((LString)node).toLNumber().getValue();
		}
		else if (node instanceof LNumber)
		{
			return _value >=
				((LNumber)node).getValue();
		}
		else
		{
			return false;
		}
	}
}
